@@ -25,6 +25,12 @@ along with this program.  If not, see
 hamzafv = {
    -- next line for ʾiʿrāb hyphen
    {a="(')(%-)([uaiUAI])", b="%1%3"},
+   -- next lines for ʾalif alone
+   {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4%4"}, --p
+   {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
    {a="A\"'", b="آ"},
@@ -78,20 +84,25 @@ hamzafv = {
    -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
-   {a="(%W)(')([ua])", b="%1أ%3"},
-   {a="(%W)(')(i)", b="%1إ%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1أ%3"},
+   --   {a="(%W)(')(i)", b="%1إ%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
    -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^say)(%')(aN)", b="%1ئ%3"},
+   {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"},
    {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
-   {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
+   -- hamzah + pr. suffix: the carrier passes into yāʾ before ī so
+   -- take out the following two lines:
+   -- {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
+   -- {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
    {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
    {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
    {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
    {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
-   {a="(u)('%p?)$", b="%1ؤْ"},
+   {a="(u)(')(%p?)$", b="%1ؤْ%3"},
    {a="(u)(')(%p?%s)", b="%1ؤْ%3"},
 -- a
    {a="(a)(')(A%p?)$", b="%1آ"},
@@ -104,27 +115,27 @@ hamzafv = {
    {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
    {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
    {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
-   {a="(a)('%p?)$", b="%1أْ"},
+   {a="(a)(')(%p?)$", b="%1أْ%3"},
    {a="(a)(')(%p?%s)", b="%1أْ%3"},
 -- i
    {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
    {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
-   {a="(i)('%p?)$", b="%1ئْ"},
+   {a="(i)(')(%p?)$", b="%1ئْ%3"},
    {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
    {a="(U)(')", b="%1ء"},
    {a="([Iy])(')", b="%1ئ"},
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
-   {a="([^uai])(')([aA])", b="%1أ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
    {a="(u)(')([uU])", b="%1ؤ%3"},
-   {a="(u)(')([aA])", b="%1ؤ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ؤ%3"},
    {a="(u)(')([iI])", b="%1ئ%3"},
-   {a="(a)(')([aA])", b="%1أ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1أ%3"},
    {a="(a)(')([uU])", b="%1ؤ%3"},
    {a="(a)(')([iI])", b="%1ئ%3"},
-   {a="(i)(')([aA])", b="%1ئ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ئ%3"},
    {a="(i)(')([uU])", b="%1ئ%3"},
    {a="(i)(')([iI])", b="%1ئ%3"},
    {a="(a)(')([^uaiUAI])", b="%1أْ%3"},
@@ -135,6 +146,12 @@ hamzafv = {
 hamzafveasy = { -- differences marked below with 'easy'
    -- next line for ʾiʿrāb hyphen
    {a="(')(%-)([uaiUAI])", b="%1%3"},
+   -- next lines for ʾalif alone
+   {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3%3"},
+   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4%4"}, --p
+   {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
    {a="A\"'", b="آ"},
@@ -190,20 +207,25 @@ hamzafveasy = { -- differences marked below with 'easy'
    -- then the 'initial' rules for the remaining cases
    {a="^(')([ua])", b="أ%2"},
    {a="^(')(i)", b="إ%2"},
-   {a="(%W)(')([ua])", b="%1أ%3"},
-   {a="(%W)(')(i)", b="%1إ%3"},
+   -- consider replacing initial %W with [%s%(%[%<%-]:
+   --   {a="(%W)(')([ua])", b="%1أ%3"},
+   --   {a="(%W)(')(i)", b="%1إ%3"},
+   {a="([%s%(%[%<%-])(')([ua])", b="%1أ%3"},
+   {a="([%s%(%[%<%-])(')(i)", b="%1إ%3"},
    -- final
    -- ^say'aN and .zim'aN are special orthographies
-   {a="(%^say)(%')(aN)", b="%1ئ%3"},
+   {a="(%^sa%.?[yY])(\"?%|?)(%')(aN)", b="%1%2ئ%4"},
    {a="(.zi?m)(%')(aN)", b="%1ئ%3"},
-   {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
-   {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
+   -- hamzah + pr. suffix: the carrier passes into yāʾ before ī so
+   -- take out the following two lines:
+   -- {a="(')(I%p?)$", b="إ%2"}, -- affix pr. 1st person sg.
+   -- {a="(')(I)(%p?%s)", b="إ%2%3"}, -- affix pr. 1st person sg.
    {a="([^uai])(')([uai]N?%p?)$", b="%1ء%3"},
    {a="([^uai])(')([uai]N?)(%p?%s)", b="%1ء%3%4"},
 -- u
    {a="(u)(')([uai]N?%p?)$", b="%1ؤ%3"},
    {a="(u)(')([uai]N?)(%p?%s)", b="%1ؤ%3%4"},
-   {a="(u)('%p?)$", b="%1ؤْ"},
+   {a="(u)(')(%p?)$", b="%1ؤْ%3"},
    {a="(u)(')(%p?%s)", b="%1ؤْ%3"},
 -- a
    {a="(a)(')(A%p?)$", b="%1آ"},
@@ -216,27 +238,27 @@ hamzafveasy = { -- differences marked below with 'easy'
    {a="(a)(')(aN)(%p?%s)", b="%1أً%4"},
    {a="(a)(')([i]N?%p?)$", b="%1إ%3"},
    {a="(a)(')([i]N?)(%p?%s)", b="%1إ%3%4"},
-   {a="(a)('%p?)$", b="%1أْ"},
+   {a="(a)(')(%p?)$", b="%1أْ%3"},
    {a="(a)(')(%p?%s)", b="%1أْ%3"},
 -- i
    {a="(i)(')([uai]N?%p?)$", b="%1ئ%3"},
    {a="(i)(')([uai]N?)(%p?%s)", b="%1ئ%3%4"},
-   {a="(i)('%p?)$", b="%1ئْ"},
+   {a="(i)(')(%p?)$", b="%1ئْ%3"},
    {a="(i)(')(%p?%s)", b="%1ئْ%3"},
 --
    -- middle
    {a="(U)(')", b="%1ء"},
    {a="([Iy])(')", b="%1ئ"},
    {a="([^uai])(')([uU])", b="%1ؤ%3"},
-   {a="([^uai])(')([aA])", b="%1أ%3"},
+   {a="([^uai])(')(%_?[aAY])", b="%1أ%3"},
    {a="([^uai])(')([iI])", b="%1ئ%3"},
    {a="(u)(')([uU])", b="%1ؤ%3"},
-   {a="(u)(')([aA])", b="%1ؤ%3"},
+   {a="(u)(')(%_?[aAY])", b="%1ؤ%3"},
    {a="(u)(')([iI])", b="%1ئ%3"},
-   {a="(a)(')([aA])", b="%1أ%3"},
+   {a="(a)(')(%_?[aAY])", b="%1أ%3"},
    {a="(a)(')([uU])", b="%1ؤ%3"},
    {a="(a)(')([iI])", b="%1ئ%3"},
-   {a="(i)(')([aA])", b="%1ئ%3"},
+   {a="(i)(')(%_?[aAY])", b="%1ئ%3"},
    {a="(i)(')([uU])", b="%1ئ%3"},
    {a="(i)(')([iI])", b="%1ئ%3"},
    {a="(a)(')([^uaiUAI])", b="%1أْ%3"},
@@ -354,12 +376,16 @@ trigraphsfv = { -- trigraphs or more
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
    {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"}, --p
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل%2"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"}, --p
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"}, -- ^n is lunar --p
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2%2"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3%3"}, --p
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"}, -- ^n is lunar --p
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل%2"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="ا%1ل%2ٱ"},
    {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"}, --p
@@ -378,15 +404,23 @@ trigraphsfv = { -- trigraphs or more
    -- art. with waṣla + lām
    {a="'l%-(l)", b="ٱل%1%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="ٱل%1%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="ٱل%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="ٱل%1%1"},
    -- li-/la- + art. + lām
    {a="l([ai])%-l%-(l)", b="ل%1%2%2"},
    -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="ٱل%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'(%^n)%-", b="ٱل"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="ٱل%1"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="ل%1ل%2%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="ل%1ل%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%2%2"},
    -- li-/la- + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="ل%1ل%3%3"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="ل%1ل%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%3%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-(\"?[uai])", b="ٱل%1ٱ"},
    -- art. with waṣla + lunar consonant (i.e. what remains)
@@ -423,12 +457,16 @@ trigraphsfveasy = { -- trigraphs or more (see 'easy' tag below for the diffs)
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
    {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
-   -- al- + solar consonant
-   {a="^(a)l%-([%_%^%.]?[tdrzsn])", b="ا%1ل%2%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsn])", b="%1ا%2ل%3%3"}, --p
-   -- assim. art. + solar consonant
-   {a="^(a)([%_%^%.]?[tdrzsn])%-", b="ا%1ل%2"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsn])%-", b="%1ا%2ل%3"}, --p
+   -- al- + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"}, -- ^n is lunar --p
+   {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2%2"},
+   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3%3"}, --p
+   -- assim. art. + solar consonant ('c' and '^n' are additional characters)
+   {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
+   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"}, -- ^n is lunar --p
+   {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل%2"},
+   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="ا%1ل%2ٱ"},
    {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1ا%2ل%3ٱ"}, --p
@@ -447,15 +485,23 @@ trigraphsfveasy = { -- trigraphs or more (see 'easy' tag below for the diffs)
    -- art. with waṣla + lām
    {a="'l%-(l)", b="ٱل%1%1"},
    -- art. with waṣla + solar consonant
-   {a="'l%-([%_%^%.]?[tdrzsn])", b="ٱل%1%1"},
+   -- ('c' and '^n' are additional characters)
+   {a="'l%-(%^n)", b="ٱل%1"}, -- ^n is lunar
+   {a="'l%-([%_%^%.]?[tdrzsnc])", b="ٱل%1%1"},
    -- li-/la- + art. + lām
    {a="l([ai])%-l%-(l)", b="ل%1%2%2"},
    -- assim. art. with waṣla + solar consonant
-   {a="'([%_%^%.]?[tdrzsn])%-", b="ٱل%1"},
+   -- ('c' and '^n' are is an additional characters)
+   {a="'(%^n)%-", b="ٱل"}, -- ^n is lunar
+   {a="'([%_%^%.]?[tdrzsnc])%-", b="ٱل%1"},
    -- li-/la- + art. + solar consonant is a special orthography
-   {a="l([ai])%-l%-([%_%^%.]?[tdrzsn])", b="ل%1ل%2%2"},
+   -- ('c' and '^n' are additional characters)
+   {a="l([ai])%-l%-(%^n)", b="ل%1ل%2"}, -- ^n is lunar
+   {a="l([ai])%-l%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%2%2"},
    -- li-/la- + assim. art. + solar consonant is a special orthography
-   {a="l([ai])%-([%_%^%.]?[tdrzsn])%-([%_%^%.]?[tdrzsn])", b="ل%1ل%3%3"},
+   -- ('c and '^n' are additional characters)
+   {a="l([ai])%-(%^n)%-(%^n)", b="ل%1ل%3"}, -- ^n is lunar
+   {a="l([ai])%-([%_%^%.]?[tdrzsnc])%-([%_%^%.]?[tdrzsnc])", b="ل%1ل%3%3"},
    -- art. with waṣla + initial unstable hamza
    {a="'l%-(\"?[uai])", b="ٱل%1ٱ"},   
    -- art. with waṣla + lunar consonant (i.e. what remains)
@@ -510,7 +556,9 @@ digraphsfvidgham = {
    {a="ئئ", b="ئّ"},
    {a="bb", b="بّ"},
    {a="BB", b="ـّ"},
-   {a="(%_)([thd])([thd])", b="%1%2|%3"},
+   {a="([%_%^%.])([tghdsz])([tghdsz])", b="%1%2|%3"},
+   -- same as above for additional characters:
+   {a="([%_%^%.])([cn])([cn])", b="%1%2|%3"},
    {a="tt", b="تّ"},
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
@@ -539,11 +587,22 @@ digraphsfvidgham = {
    {a="hh", b="هّ"},
    {a="ww", b="وّ"},
    {a="yy", b="يّ"},
+   {a="%.y%.y", b="ىّ"},
+   -- additional characters + šaddah (begin)
+   {a="pp", b="پّ"},
+   {a="vv", b="ڤّ"},
+   {a="gg", b="گّ"},
+   {a="%^c%^c", b="چّ"},
+   {a="%^z%^z", b="ژّ"},
+   {a="%^n%^n", b="ڭّ"},
+   -- additional characters + šaddah (end)
    -- sukūn begin
-   -- first, take out hyphen if any:
+   -- first, take out hyphen if any (next two lines):
+   {a="([ai]Y)%-([uaiUAI])", b="%1%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
-   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?)$", b="%1ْ%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
+   {a="([ai]Y)([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    -- take out sukūn in cases of assimilation
    {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
@@ -565,6 +624,12 @@ digraphsfvidgham = {
    {a="%.t", b="ط"},
    {a="%.z", b="ظ"},
    {a="%.g", b="غ"},
+   {a="%.y", b="ى"},
+   -- additional characters (begin)
+   {a="%^c", b="چ"},
+   {a="%^z", b="ژ"},
+   {a="%^n", b="ڭ"},
+   -- additional characters (end)
    {a="(U)(A)", b="%1ا"},
    {a="WA", b="وْا"},
    {a="(a)W\"", b="%1وا"},
@@ -618,7 +683,9 @@ digraphsfv = {
    {a="ئئ", b="ئّ"},
    {a="bb", b="بّ"},
    {a="BB", b="ـّ"},
-   {a="(%_)([thd])([thd])", b="%1%2|%3"},
+   {a="([%_%^%.])([tghdsz])([tghdsz])", b="%1%2|%3"},
+   -- same as above for additional characters:
+   {a="([%_%^%.])([cn])([cn])", b="%1%2|%3"},
    {a="tt", b="تّ"},
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
@@ -647,11 +714,22 @@ digraphsfv = {
    {a="hh", b="هّ"},
    {a="ww", b="وّ"},
    {a="yy", b="يّ"},
+   {a="%.y%.y", b="ىّ"},
+   -- additional characters + šaddah (begin)
+   {a="pp", b="پّ"},
+   {a="vv", b="ڤّ"},
+   {a="gg", b="گّ"},
+   {a="%^c%^c", b="چّ"},
+   {a="%^z%^z", b="ژّ"},
+   {a="%^n%^n", b="ڭّ"},
+   -- additional characters + šaddah (end)
    -- sukūn begin
-   -- first, take out hyphen if any:
+   -- first, take out hyphen if any (next two lines):
+   {a="([ai]Y)%-([uaiUAI])", b="%1%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
-   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?)$", b="%1ْ%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
+   {a="([ai]Y)([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    -- take out sukūn in cases of assimilation
 --   {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
@@ -673,6 +751,12 @@ digraphsfv = {
    {a="%.t", b="ط"},
    {a="%.z", b="ظ"},
    {a="%.g", b="غ"},
+   {a="%.y", b="ى"},
+   -- additional characters (begin)
+   {a="%^c", b="چ"},
+   {a="%^z", b="ژ"},
+   {a="%^n", b="ڭ"},
+   -- additional characters (end)
    {a="(U)(A)", b="%1ا"},
    {a="WA", b="وْا"},
    {a="(a)W\"", b="%1وا"},
@@ -726,7 +810,9 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="ئئ", b="ئّ"},
    {a="bb", b="بّ"},
    {a="BB", b="ـّ"},
-   {a="(%_)([thd])([thd])", b="%1%2|%3"},
+   {a="([%_%^%.])([tghdsz])([tghdsz])", b="%1%2|%3"},
+   -- same as above for additional characters:
+   {a="([%_%^%.])([cn])([cn])", b="%1%2|%3"},
    {a="tt", b="تّ"},
    {a="%_t%_t", b="ثّ"},
    {a="jj", b="جّ"},
@@ -755,15 +841,26 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="hh", b="هّ"},
    {a="ww", b="وّ"},
    {a="yy", b="يّ"},
+   {a="%.y%.y", b="ىّ"},
+   -- additional characters + šaddah (begin)
+   {a="pp", b="پّ"},
+   {a="vv", b="ڤّ"},
+   {a="gg", b="گّ"},
+   {a="%^c%^c", b="چّ"},
+   {a="%^z%^z", b="ژّ"},
+   {a="%^n%^n", b="ڭّ"},
+   -- additional characters + šaddah (end)
    -- sukūn begin ('easy' needs these rules to be taken out); but
    -- first take out every previously generated sukūn by hamza rules,
    -- so there be no need to edit them:
    {a="ْ", b=""}, 
-   -- first, take out hyphen if any:
+   -- first, take out hyphen if any (next two lines):
+--   {a="([ai]Y)%-([uaiUAI])", b="%1%2"},
 --   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])%-([uaiUAI])", b="%1%2"},
---   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy]%p?)$", b="%1ْ"},
+--   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?)$", b="%1ْ%2"},
 --   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])(%p?%s)", b="%1ْ%2"},
---   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIًٌٍ])", b="%1ْ%2"},
+--   {a="([ai]Y)([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
+--   {a="([%_%^%.]?[Bbtjghxdrzs%`fqklmnwy])([%_]?[^%_uaiUAIYًٌٍ])", b="%1ْ%2"},
    -- take out sukūn in cases of assimilation
 --   {a="(n)(ْ)(%s)(ر)", b="%1%3%4"},
 --   {a="(n)(ْ)(%s)(و)", b="%1%3%4"},
@@ -784,6 +881,12 @@ digraphsfveasy = { -- see the differences under 'easy' marker below
    {a="%.t", b="ط"},
    {a="%.z", b="ظ"},
    {a="%.g", b="غ"},
+   {a="%.y", b="ى"},
+   -- additional characters (begin)
+   {a="%^c", b="چ"},
+   {a="%^z", b="ژ"},
+   {a="%^n", b="ڭ"},
+   -- additional characters (end)
    {a="(U)(A)", b="%1ا"},
    {a="WA", b="وْا"},
    {a="(a)W\"", b="%1وا"},
@@ -821,6 +924,11 @@ singlefv = {
    {a="w", b="و"},
    {a="y", b="ي"},
    {a="T", b="ة"},
+   -- additional characters (begin)
+   {a="p", b="پ"},
+   {a="v", b="ڤ"},
+   {a="g", b="گ"},
+   -- additional characters (end)
    {a="\"$", b=""},
    {a="\"(%W)", b="%1"},
    {a="\"([^uaiUAI])", b="%1"},
@@ -849,6 +957,11 @@ singlefveasy = { -- see the differences under 'easy' tag below
    {a="w", b="و"},
    {a="y", b="ي"},
    {a="T", b="ة"},
+   -- additional characters (begin)
+   {a="p", b="پ"},
+   {a="v", b="ڤ"},
+   {a="g", b="گ"},
+   -- additional characters (end)
    -- easy (begin): \" needs to put back the sukūn
    {a="\"$", b="ْ"},
    {a="\"(%W)", b="ْ%1"},
